@@ -1,4 +1,5 @@
 package Catalyst::Helper::View::ByCode;
+$Catalyst::Helper::View::ByCode::VERSION = '0.10';
 
 use strict;
 
@@ -6,6 +7,10 @@ use strict;
 
 Catalyst::Helper::View::ByCode - Helper for ByCode Views
 
+=head1 VERSION
+
+version 0.10
+
 =head1 SYNOPSIS
 
     script/create.pl view ByCode ByCode
@@ -48,9 +53,8 @@ __DATA__
 __compclass__
 package [% class %];
 
-use strict;
-use warnings;
-use parent 'Catalyst::View::ByCode';
+use Moose;
+BEGIN { extends 'Catalyst::View::ByCode' }
 
 __PACKAGE__->config(
     # # Change default
@@ -59,7 +63,7 @@ __PACKAGE__->config(
     # # Set the location for .pl files
     # root_dir => 'root/bycode',
     # 
-    # # This is your wrapper template located in the 'root/src'
+    # # This is your wrapper template located in the 'root_dir'
     # wrapper => 'wrapper.pl',
     #
     # # specify packages to use in every template
@@ -94,4 +98,4 @@ it under the same terms as Perl itself.
 
 =cut
 
-1;
+1;
\ No newline at end of file
